@@ -36,7 +36,11 @@
 struct _NotifyStack {
 	NotifyDaemon* daemon;
 	GdkScreen* screen;
+#if GTK_CHECK_VERSION (3, 22, 0)
+	GdkMonitor *monitor;
+#else
 	guint monitor;
+#endif
 	NotifyStackLocation location;
 	GList* windows;
 	guint update_id;
@@ -71,8 +75,9 @@ get_work_area (NotifyStack  *stack,
         /* Defaults in case of error */
         rect->x = 0;
         rect->y = 0;
-        rect->width = gdk_screen_get_width (stack->screen);
-        rect->height = gdk_screen_get_height (stack->screen);
+
+        gdk_window_get_geometry (gdk_screen_get_root_window (stack->screen), NULL, NULL,
+                                 &rect->width, &rect->height);
 
         if (workarea == None)
                 return FALSE;
@@ -189,17 +194,48 @@ translate_coordinates (NotifyStackLocation stack_location,
         }
 }
 
+#if GTK_CHECK_VERSION(3, 22, 0)
+static int
+_gtk_get_monitor_num (GdkMonitor *monitor)
+{
+        GdkDisplay *display;
+        int n_monitors, i;
+
+        display = gdk_monitor_get_display(monitor);
+        n_monitors = gdk_display_get_n_monitors(display);
+
+        for(i = 0; i < n_monitors; i++)
+        {
+                if (gdk_display_get_monitor(display, i) == monitor) return i;
+        }
+
+        return -1;
+}
+#endif
+
 NotifyStack *
 notify_stack_new (NotifyDaemon       *daemon,
                   GdkScreen          *screen,
+#if GTK_CHECK_VERSION (3, 22, 0)
+                  GdkMonitor         *monitor,
+#else
                   guint               monitor,
+#endif
                   NotifyStackLocation location)
 {
         NotifyStack    *stack;
+#if GTK_CHECK_VERSION (3, 22, 0)
+        GdkDisplay     *display;
 
+        display = gdk_screen_get_display (screen);
+#endif
         g_assert (daemon != NULL);
         g_assert (screen != NULL && GDK_IS_SCREEN (screen));
+#if GTK_CHECK_VERSION (3, 22, 0)
+        g_assert ((guint)_gtk_get_monitor_num (monitor) < (guint)gdk_display_get_n_monitors (display));
+#else
         g_assert (monitor < (guint)gdk_screen_get_n_monitors (screen));
+#endif
         g_assert (location != NOTIFY_STACK_LOCATION_UNKNOWN);
 
         stack = g_new0 (NotifyStack, 1);
@@ -272,9 +308,13 @@ notify_stack_shift_notifications (NotifyStack *stack,
         int             n_wins;
 
         get_work_area (stack, &workarea);
+#if GTK_CHECK_VERSION (3, 22, 0)
+	gdk_monitor_get_geometry (stack->monitor, &monitor);
+#else
         gdk_screen_get_monitor_geometry (stack->screen,
                                          stack->monitor,
                                          &monitor);
+#endif
         gdk_rectangle_intersect (&monitor, &workarea, &workarea);
 
         add_padding_to_rect (&workarea);
@@ -301,11 +341,7 @@ notify_stack_shift_notifications (NotifyStack *stack,
                 GtkRequisition  req;
 
                 if (nw == NULL || nw2 != nw) {
-#if GTK_CHECK_VERSION (3, 0, 0)
                         gtk_widget_get_preferred_size (GTK_WIDGET (nw2), NULL, &req);
-#else
-                        gtk_widget_size_request (GTK_WIDGET (nw2), &req);
-#endif
 
                         translate_coordinates (stack->location,
                                                &workarea,
@@ -366,11 +402,7 @@ void notify_stack_add_window(NotifyStack* stack, GtkWindow* nw, gboolean new_not
 	GtkRequisition  req;
 	gint            x, y;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	gtk_widget_get_preferred_size(GTK_WIDGET(nw), NULL, &req);
-#else
-	gtk_widget_size_request(GTK_WIDGET(nw), &req);
-#endif
 	notify_stack_shift_notifications(stack, nw, NULL, req.width, req.height + NOTIFY_STACK_SPACING, &x, &y);
 	theme_move_notification(nw, x, y);
 
