@@ -64,7 +64,8 @@ typedef struct
 	ArrowParameters arrow;
 
 	gboolean composited;
-	
+	gboolean action_icons;
+
 	int width;
 	int height;
 	int last_width;
@@ -136,8 +137,8 @@ get_notification_arrow_type(GtkWidget *nw)
 	WindowData *windata = g_object_get_data(G_OBJECT(nw), "windata");
 	int screen_height;
 
-	screen_height = gdk_screen_get_height(
-		gdk_window_get_screen(gtk_widget_get_window(nw)));
+	gdk_window_get_geometry (gdk_screen_get_root_window (gdk_window_get_screen (gtk_widget_get_window (nw))),
+				 NULL, NULL, NULL, &screen_height);
 
 	if (windata->arrow.position.y + windata->height + DEFAULT_ARROW_HEIGHT >
 		screen_height)
@@ -158,8 +159,8 @@ set_arrow_parameters (WindowData *windata)
 	int x,y;
 	GtkArrowType arrow_type;
 
-	screen_width = gdk_screen_get_width(
-		gdk_window_get_screen(gtk_widget_get_window(windata->win)));
+	gdk_window_get_geometry (gdk_screen_get_root_window (gdk_window_get_screen (gtk_widget_get_window (windata->win))),
+				 NULL, NULL, &screen_width, NULL);
 
 	/* Set arrow offset */
 	GtkAllocation alloc;
@@ -974,20 +975,26 @@ add_notification_action(GtkWindow *nw, const char *text, const char *key,
 		gtk_widget_show(windata->actions_box);
 		update_content_hbox_visibility(windata);
 
-		windata->pie_countdown = gtk_drawing_area_new();
-		gtk_widget_set_halign (windata->pie_countdown, GTK_ALIGN_END);
-		gtk_widget_show(windata->pie_countdown);
+		/* Don't try to re-add a pie_countdown */
+		if (!windata->pie_countdown) {
+			windata->pie_countdown = gtk_drawing_area_new();
+			gtk_widget_set_halign (windata->pie_countdown, GTK_ALIGN_END);
+			gtk_widget_show(windata->pie_countdown);
 
-		gtk_box_pack_end (GTK_BOX (windata->actions_box), windata->pie_countdown, FALSE, TRUE, 0);
-		gtk_widget_set_size_request(windata->pie_countdown,
-									PIE_WIDTH, PIE_HEIGHT);
-		g_signal_connect(G_OBJECT(windata->pie_countdown), "draw",
-						 G_CALLBACK(countdown_expose_cb), windata);
+			gtk_box_pack_end (GTK_BOX (windata->actions_box), windata->pie_countdown, FALSE, TRUE, 0);
+			gtk_widget_set_size_request(windata->pie_countdown,
+						    PIE_WIDTH, PIE_HEIGHT);
+			g_signal_connect(G_OBJECT(windata->pie_countdown), "draw",
+					 G_CALLBACK(countdown_expose_cb), windata);
+		}
+	}
+
+	if (windata->action_icons) {
+		button = gtk_button_new_from_icon_name(key, GTK_ICON_SIZE_BUTTON);
+		goto add_button;
 	}
 
 	button = gtk_button_new();
-	gtk_widget_show(button);
-	gtk_box_pack_start(GTK_BOX(windata->actions_box), button, FALSE, FALSE, 0);
 
 	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
 	gtk_widget_show(hbox);
@@ -1023,11 +1030,17 @@ add_notification_action(GtkWindow *nw, const char *text, const char *key,
 	gtk_label_set_markup(GTK_LABEL(label), buf);
 	g_free(buf);
 
+add_button:
+	gtk_widget_show(button);
+	gtk_box_pack_start(GTK_BOX(windata->actions_box), button, FALSE, FALSE, 0);
+
 	g_object_set_data(G_OBJECT(button), "_nw", nw);
 	g_object_set_data_full(G_OBJECT(button),
 						   "_action_key", g_strdup(key), g_free);
 	g_signal_connect(G_OBJECT(button), "button-release-event",
 					 G_CALLBACK(action_clicked_cb), cb);
+
+	gtk_widget_show_all(windata->actions_box);
 }
 
 /* Clear notification actions */
@@ -1084,11 +1097,12 @@ void
 set_notification_hints(GtkWindow *nw, GHashTable *hints)
 {
 	WindowData *windata = g_object_get_data(G_OBJECT(nw), "windata");
-	GValue *value;
+	GValue *value = NULL, *icon_value = NULL;
 
 	g_assert(windata != NULL);
 
 	value = (GValue *)g_hash_table_lookup(hints, "urgency");
+	icon_value = (GValue *)g_hash_table_lookup(hints, "action-icons");
 
 	if (value != NULL && G_VALUE_HOLDS_UCHAR(value))
 	{
@@ -1100,6 +1114,12 @@ set_notification_hints(GtkWindow *nw, GHashTable *hints)
 			gtk_window_set_title(GTK_WINDOW(nw), "Notification");
 		}
 	}
+
+	/* Determine if action-icons have been requested */
+	if (icon_value != NULL && G_VALUE_HOLDS_BOOLEAN(icon_value))
+	{
+		windata->action_icons = g_value_get_boolean(icon_value);
+	}
 }
 
 /* Notification tick */
