@@ -45,6 +45,7 @@ typedef struct {
 
 	gboolean has_arrow;
 	gboolean composited;
+	gboolean action_icons;
 
 	int point_x;
 	int point_y;
@@ -265,13 +266,24 @@ static GtkArrowType get_notification_arrow_type(GtkWidget* nw)
 	WindowData*     windata;
 	GdkScreen*      screen;
 	GdkRectangle    monitor_geometry;
+#if GTK_CHECK_VERSION (3, 22, 0)
+	GdkDisplay*     display;
+	GdkMonitor*     monitor;
+#else
 	int             monitor;
+#endif
 
 	windata = g_object_get_data(G_OBJECT(nw), "windata");
 
 	screen = gdk_window_get_screen(GDK_WINDOW( gtk_widget_get_window(nw)));
+#if GTK_CHECK_VERSION (3, 22, 0)
+	display = gdk_screen_get_display (screen);
+	monitor = gdk_display_get_monitor_at_point (display, windata->point_x, windata->point_y);
+	gdk_monitor_get_geometry (monitor, &monitor_geometry);
+#else
 	monitor = gdk_screen_get_monitor_at_point(screen, windata->point_x, windata->point_y);
 	gdk_screen_get_monitor_geometry(screen, monitor, &monitor_geometry);
+#endif
 
 	if (windata->point_y - monitor_geometry.y + windata->height + DEFAULT_ARROW_HEIGHT > monitor_geometry.height)
 	{
@@ -306,15 +318,26 @@ static void create_border_with_arrow(GtkWidget* nw, WindowData* windata)
 	int             arrow_offset = DEFAULT_ARROW_OFFSET;
 	GdkPoint*       shape_points = NULL;
 	int             i = 0;
+#if GTK_CHECK_VERSION (3, 22, 0)
+	GdkMonitor*     monitor;
+	GdkDisplay*     display;
+#else
 	int             monitor;
+#endif
 	GdkRectangle    monitor_geometry;
 
 	width = windata->width;
 	height = windata->height;
 
 	screen = gdk_window_get_screen(GDK_WINDOW(gtk_widget_get_window(nw)));
+#if GTK_CHECK_VERSION (3, 22, 0)
+	display = gdk_screen_get_display (screen);
+	monitor = gdk_display_get_monitor_at_point (display, windata->point_x, windata->point_y);
+	gdk_monitor_get_geometry (monitor, &monitor_geometry);
+#else
 	monitor = gdk_screen_get_monitor_at_point(screen, windata->point_x, windata->point_y);
 	gdk_screen_get_monitor_geometry(screen, monitor, &monitor_geometry);
+#endif
 
 	windata->num_border_points = 5;
 
@@ -796,27 +819,32 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	return GTK_WINDOW(win);
 }
 
-void set_notification_hints(GtkWindow* nw, GHashTable* hints)
+void set_notification_hints(GtkWindow *nw, GHashTable *hints)
 {
-	WindowData* windata = g_object_get_data(G_OBJECT(nw), "windata");
+	WindowData *windata = g_object_get_data(G_OBJECT(nw), "windata");
+	GValue *value = NULL, *icon_value = NULL;
 
 	g_assert(windata != NULL);
 
-	GValue* value = (GValue*) g_hash_table_lookup(hints, "urgency");
+	value = (GValue *)g_hash_table_lookup(hints, "urgency");
+	icon_value = (GValue *)g_hash_table_lookup(hints, "action-icons");
 
 	if (value != NULL && G_VALUE_HOLDS_UCHAR(value))
 	{
 		windata->urgency = g_value_get_uchar(value);
 
-		if (windata->urgency == URGENCY_CRITICAL)
-		{
+		if (windata->urgency == URGENCY_CRITICAL) {
 			gtk_window_set_title(GTK_WINDOW(nw), "Critical Notification");
-		}
-		else
-		{
+		} else {
 			gtk_window_set_title(GTK_WINDOW(nw), "Notification");
 		}
 	}
+
+	/* Determine if action-icons have been requested */
+	if (icon_value != NULL && G_VALUE_HOLDS_BOOLEAN(icon_value))
+	{
+		windata->action_icons = g_value_get_boolean(icon_value);
+	}
 }
 
 void set_notification_timeout(GtkWindow* nw, glong timeout)
@@ -1016,18 +1044,26 @@ void add_notification_action(GtkWindow* nw, const char* text, const char* key, A
 		gtk_widget_show(windata->actions_box);
 		update_content_hbox_visibility(windata);
 
-		windata->pie_countdown = gtk_drawing_area_new();
-		gtk_widget_set_halign (windata->pie_countdown, GTK_ALIGN_END);
-		gtk_widget_show(windata->pie_countdown);
-		gtk_box_pack_end(GTK_BOX(windata->actions_box), windata->pie_countdown, FALSE, TRUE, 0);
-		gtk_widget_set_size_request(windata->pie_countdown, PIE_WIDTH, PIE_HEIGHT);
-		g_signal_connect (G_OBJECT (windata->pie_countdown), "draw", G_CALLBACK (on_countdown_draw), windata);
+		/* Don't try to re-add a pie_countdown */
+		if (!windata->pie_countdown) {
+			windata->pie_countdown = gtk_drawing_area_new();
+			gtk_widget_set_halign (windata->pie_countdown, GTK_ALIGN_END);
+			gtk_widget_show(windata->pie_countdown);
+
+			gtk_box_pack_end (GTK_BOX (windata->actions_box), windata->pie_countdown, FALSE, TRUE, 0);
+			gtk_widget_set_size_request(windata->pie_countdown,
+						    PIE_WIDTH, PIE_HEIGHT);
+			g_signal_connect(G_OBJECT(windata->pie_countdown), "draw",
+					 G_CALLBACK(on_countdown_draw), windata);
+		}
+	}
+
+	if (windata->action_icons) {
+		button = gtk_button_new_from_icon_name(key, GTK_ICON_SIZE_BUTTON);
+		goto add_button;
 	}
 
 	button = gtk_button_new();
-	gtk_widget_show(button);
-	gtk_box_pack_start(GTK_BOX(windata->actions_box), button, FALSE, FALSE, 0);
-
 	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
 	gtk_widget_show(hbox);
 	gtk_container_add(GTK_CONTAINER(button), hbox);
@@ -1060,9 +1096,15 @@ void add_notification_action(GtkWindow* nw, const char* text, const char* key, A
 	gtk_label_set_markup(GTK_LABEL(label), buf);
 	g_free(buf);
 
+add_button:
+	gtk_widget_show(button);
+	gtk_box_pack_start(GTK_BOX(windata->actions_box), button, FALSE, FALSE, 0);
+
 	g_object_set_data(G_OBJECT(button), "_nw", nw);
 	g_object_set_data_full(G_OBJECT(button), "_action_key", g_strdup(key), g_free);
 	g_signal_connect(G_OBJECT(button), "button-release-event", G_CALLBACK(action_clicked_cb), cb);
+
+	gtk_widget_show_all(windata->actions_box);
 }
 
 void clear_notification_actions(GtkWindow* nw)
